@@ -8,7 +8,9 @@ use Mojo::Pg::Migrations;
 use Mojo::Pg::PubSub;
 use Mojo::URL;
 use Scalar::Util 'weaken';
+use SQL::Abstract;
 
+has abstract => sub { SQL::Abstract->new(quote_char => '"') };
 has [qw(auto_migrate search_path)];
 has database_class  => 'Mojo::Pg::Database';
 has dsn             => 'dbi:Pg:';
@@ -28,7 +30,7 @@ has pubsub => sub {
   return $pubsub;
 };
 
-our $VERSION = '2.35';
+our $VERSION = '3.0';
 
 sub db { $_[0]->database_class->new(dbh => $_[0]->_dequeue, pg => $_[0]) }
 
@@ -103,8 +105,10 @@ Mojo::Pg - Mojolicious ♥ PostgreSQL
 
   use Mojo::Pg;
 
-  # Select the server version
+  # Use a PostgreSQL connection string for configuration
   my $pg = Mojo::Pg->new('postgresql://postgres@/test');
+
+  # Select the server version
   say $pg->db->query('select version() as version')->hash->{version};
 
   # Use migrations to create a table
@@ -118,8 +122,16 @@ Mojo::Pg - Mojolicious ♥ PostgreSQL
   # Use migrations to drop and recreate the table
   $pg->migrations->migrate(0)->migrate;
 
-  # Insert a few rows
+  # Get a database handle from the cache for multiple queries
   my $db = $pg->db;
+
+  # Use SQL::Abstract to generate simple CRUD queries for you
+  $db->insert('names', {name => 'Isabel'});
+  say $db->select('names', ['id'], {name => 'Isabel'})->hash->{id};
+  $db->update('names', {name => 'Bel'}, {name => 'Isabel'});
+  $db->delete('names', {name => 'Bel'});
+
+  # Insert a few rows
   $db->query('insert into names (name) values (?)', 'Sara');
   $db->query('insert into names (name) values (?)', 'Stefan');
 
@@ -262,6 +274,17 @@ Which get integrated into your application with helpers.
 
   app->start;
 
+=head1 EXAMPLES
+
+This distribution also contains two great
+L<example applications|https://github.com/kraih/mojo-pg/tree/master/examples/>
+you can use for inspiration. The minimal
+L<chat|https://github.com/kraih/mojo-pg/tree/master/examples/chat.pl>
+application will show you how to scale WebSockets to multiple servers, and the
+well-structured
+L<blog|https://github.com/kraih/mojo-pg/tree/master/examples/blog> application
+how to apply the MVC design pattern in practice.
+
 =head1 EVENTS
 
 L<Mojo::Pg> inherits all events from L<Mojo::EventEmitter> and can emit the
@@ -285,6 +308,16 @@ Emitted when a new database connection has been established.
 
 L<Mojo::Pg> implements the following attributes.
 
+=head2 abstract
+
+  my $abstract = $pg->abstract;
+  $pg          = $pg->abstract(SQL::Abstract->new);
+
+L<SQL::Abstract> object used to generate CRUD queries for L<Mojo::Pg::Database>.
+
+  # Generate statements and bind values
+  my($stmt, @bind) = $pg->abstract->select('names');
+
 =head2 auto_migrate
 
   my $bool = $pg->auto_migrate;
@@ -398,7 +431,7 @@ problems like connection timeouts gracefully by holding on to it only for short
 amounts of time.
 
   # Add up all the money
-  say $pg->db->query('select * from accounts')
+  say $pg->db->select('accounts')
     ->hashes->reduce(sub { $a->{money} + $b->{money} });
 
 =head2 from_string
@@ -439,6 +472,15 @@ L</"from_string"> if necessary.
   # Customize configuration further
   my $pg = Mojo::Pg->new->dsn('dbi:Pg:service=foo');
 
+=head1 DEBUGGING
+
+You can set the C<DBI_TRACE> environment variable to get some advanced
+diagnostics information printed by L<DBI>.
+
+  DBI_TRACE=1
+  DBI_TRACE=15
+  DBI_TRACE=SQL
+
 =head1 REFERENCE
 
 This is the class hierarchy of the L<Mojo::Pg> distribution.
