@@ -7,9 +7,18 @@ use Mojo::IOLoop;
 use Mojo::JSON 'to_json';
 use Mojo::Pg::Results;
 use Mojo::Pg::Transaction;
+use Mojo::Util 'monkey_patch';
 use Scalar::Util 'weaken';
 
 has [qw(dbh pg)];
+has results_class => 'Mojo::Pg::Results';
+
+for my $name (qw(delete insert select update)) {
+  monkey_patch __PACKAGE__, lc $name, sub {
+    my ($self, @cb) = (shift, ref $_[-1] eq 'CODE' ? pop : ());
+    return $self->query($self->pg->abstract->$name(@_), @cb);
+  };
+}
 
 sub DESTROY {
   my $self = shift;
@@ -76,12 +85,23 @@ sub query {
   $attrs{pg_async}                  = PG_ASYNC if $cb;
   my $sth = $self->dbh->prepare_cached($query, \%attrs, 3);
   local $sth->{HandleError} = sub { $_[0] = shortmess $_[0]; 0 };
-  $sth->execute(map { _json($_) ? to_json $_->{json} : $_ } @_);
+
+  for (my $i = 0; $#_ >= $i; $i++) {
+    my ($param, $attrs) = ($_[$i], {});
+    if (ref $param eq 'HASH') {
+      if (exists $param->{json}) { $param = to_json $param->{json} }
+      elsif (exists $param->{type} && exists $param->{value}) {
+        ($attrs->{pg_type}, $param) = @{$param}{qw(type value)};
+      }
+    }
+    $sth->bind_param($i + 1, $param, $attrs);
+  }
+  $sth->execute;
 
   # Blocking
   unless ($cb) {
     $self->_notifications;
-    return Mojo::Pg::Results->new(sth => $sth);
+    return $self->results_class->new(sth => $sth);
   }
 
   # Non-blocking
@@ -105,8 +125,6 @@ sub unlisten {
   return $self;
 }
 
-sub _json { ref $_[0] eq 'HASH' && exists $_[0]{json} }
-
 sub _notifications {
   my $self = shift;
   my $dbh  = $self->dbh;
@@ -141,7 +159,7 @@ sub _watch {
       my $result = do { local $dbh->{RaiseError} = 0; $dbh->pg_result };
       my $err = defined $result ? undef : $dbh->errstr;
 
-      $self->$cb($err, Mojo::Pg::Results->new(sth => $sth));
+      $self->$cb($err, $self->results_class->new(sth => $sth));
       $self->_unwatch unless $self->{waiting} || $self->is_listening;
     }
   )->watch($self->{handle}, 1, 0);
@@ -213,6 +231,14 @@ L<DBD::Pg> database handle used for all queries.
 
 L<Mojo::Pg> object this database belongs to.
 
+=head2 results_class
+
+  my $class = $db->results_class;
+  $db       = $db->results_class('MyApp::Results');
+
+Class to be used by L</"query">, defaults to L<Mojo::Pg::Results>. Note that
+this class needs to have already been loaded before L</"query"> is called.
+
 =head1 METHODS
 
 L<Mojo::Pg::Database> inherits all methods from L<Mojo::EventEmitter> and
@@ -229,12 +255,26 @@ L<Mojo::Pg::Transaction/"commit"> has been called before it is destroyed.
   # Insert rows in a transaction
   eval {
     my $tx = $db->begin;
-    $db->query('insert into frameworks values (?)', 'Catalyst');
-    $db->query('insert into frameworks values (?)', 'Mojolicious');
+    $db->insert('frameworks', {name => 'Catalyst'});
+    $db->insert('frameworks', {name => 'Mojolicious'});
     $tx->commit;
   };
   say $@ if $@;
 
+=head2 delete
+
+  my $results = $db->delete($table, \%where);
+
+Generate a C<DELETE> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->delete(some_table => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 disconnect
 
   $db->disconnect;
@@ -252,6 +292,20 @@ to be used as an operator.
   $db->dollar_only->query('select * from foo where bar ? $1', 'baz')
     ->expand->hashes->map(sub { $_->{bar}{baz} })->join("\n")->say;
 
+=head2 insert
+
+  my $results = $db->insert($table, \@values || \%fieldvals, \%options);
+
+Generate an C<INSERT> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->insert(some_table => {foo => 'bar'} => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 is_listening
 
   my $bool = $db->is_listening;
@@ -291,10 +345,10 @@ Check database connection.
   my $results = $db->query('select ?::json as foo', {json => {bar => 'baz'}});
 
 Execute a blocking L<SQL|http://www.postgresql.org/docs/current/static/sql.html>
-statement and return a L<Mojo::Pg::Results> object with the results. The
-L<DBD::Pg> statement handle will be automatically reused when it is not active
-anymore, to increase the performance of future queries. You can also append a
-callback to perform operations non-blocking.
+statement and return a results object based on L</"results_class"> with the
+query results. The L<DBD::Pg> statement handle will be automatically reused when
+it is not active anymore, to increase the performance of future queries. You can
+also append a callback to perform operations non-blocking.
 
   $db->query('insert into foo values (?, ?, ?)' => @values => sub {
     my ($db, $err, $results) = @_;
@@ -311,6 +365,27 @@ of the types C<json> and C<jsonb> with L<Mojo::JSON/"from_json"> to Perl values.
   $db->query('select ?::jsonb as foo', {json => {bar => 'I ♥ Mojolicious!'}})
     ->expand->hash->{foo}{bar};
 
+Hash reference arguments containing values named C<type> and C<value>, can be
+used to bind specific L<DBD::Pg> data types to placeholders.
+
+  # Insert binary data
+  use DBD::Pg ':pg_types';
+  $db->query('insert into bar values (?)', {type => PG_BYTEA, value => $bytes});
+
+=head2 select
+
+  my $results = $db->select($source, $fields, $where, $order);
+
+Generate a C<SELECT> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->select(some_table => ['foo'] => {bar => 'yada'} => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head2 tables
 
   my $tables = $db->tables;
@@ -328,6 +403,20 @@ user and not internal, as an array reference.
 
 Unsubscribe from a channel, C<*> can be used to unsubscribe from all channels.
 
+=head2 update
+
+  my $results = $db->update($table, \%fieldvals, \%where);
+
+Generate an C<UPDATE> statement with L<Mojo::Pg/"abstract"> (usually an
+L<SQL::Abstract> object) and execute it with L</"query">. You can also append a
+callback to perform operations non-blocking.
+
+  $db->update(some_table => {foo => 'baz'} => {foo => 'bar'} => sub {
+    my ($db, $err, $results) = @_;
+    ...
+  });
+  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
+
 =head1 SEE ALSO
 
 L<Mojo::Pg>, L<Mojolicious::Guides>, L<http://mojolicious.org>.
